﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;

namespace ListViewSample
{
    public class SfPopUpView : ContentView
    {
        /// <summary>
        /// Defines constants that specify the location for pop-up when it is relative to another view.
        /// </summary>
        public enum PopupLocation
        {
            /// <summary>
            /// Constant to display the pop-up on top of the specified view
            /// </summary>
            Top,

            /// <summary>
            /// Constant to display the pop-up below the specified view
            /// </summary>
            Bottom,

            /// <summary>
            /// Constant to display the pop-up on left of the specified view
            /// </summary>
            Left,

            /// <summary>
            /// Constant to display the pop-up to the right of the specified view
            /// </summary>
            Right
        }

        /// <summary>
        /// The content for the SfPopUpView
        /// </summary>
        private View content;

        /// <summary>
        /// The pop-up view that has to be displayed.
        /// </summary>
        private View popup;

        private readonly RelativeLayout popUpContentLayout;

        /// <summary>
        /// Initializes a new instance of the <see cref="SfPopUpView"/> class.
        /// </summary>
        public SfPopUpView()
        {
            base.Content = this.popUpContentLayout = new RelativeLayout();
        }


        /// <summary>
        /// Identifies the Content bindable property.
        /// </summary>
        public new static readonly BindableProperty ContentProperty =
                  BindableProperty.Create("Content", typeof(View), typeof(SfPopUpView), null, BindingMode.OneWay, null, propertyChanged: OnContentChanged);

        /// <summary>
        /// Occurs when the Content property changes.
        /// </summary>
        /// <param name="bindable">The SfPopUpView as a bindable object.</param>
        /// <param name="oldValue">The old value.</param>
        /// <param name="newValue">The new value.</param>
        private static void OnContentChanged(BindableObject bindable, object oldValue, object newValue)
        {
            var view = bindable as SfPopUpView;
            view.UpdateContent((View)newValue);
        }

        /// <summary>Gets or sets the content of the ContentView.</summary>
        /// <value>A <see cref="T:Xamarin.Forms.View" /> that contains the content.</value>
        /// <remarks />
        public new View Content
        {
            get
            {
                return this.content;
            }
            set
            {
                //var oldValue = this.content;
                base.SetValue(ContentView.ContentProperty, value);
                //OnContentChanged(this, oldValue, value);
            }
        }

        /// <summary>
        /// Tweaks to add the content of the pop-up as a child of the relative layout.
        /// </summary>
        /// <param name="value"></param>
        private void UpdateContent(View value)
        {
            if (this.content != null)
            {
                this.popUpContentLayout.Children.Remove(this.content);
            }

            this.content = value;
            this.popUpContentLayout.Children.Add(this.content, () => this.Bounds);
        }

        /// <summary>
        /// Gets a value indicating whether this instance is pop-up active.
        /// </summary>
        /// <value><c>true</c> if this instance is pop-up active; otherwise, <c>false</c>.</value>
        public bool IsPopupActive
        {
            get { return this.popup != null; }
        }

        /// <summary>
        /// Shows the pop-up in the center of its parent.
        /// </summary>
        /// <param name="popupView">The pop-up view.</param>
        public void ShowPopup(View popupView)
        {
            this.ShowPopup
            (
                popupView,
                Constraint.RelativeToParent(p => (this.Width - this.popup.WidthRequest) / 2),
                Constraint.RelativeToParent(p => (this.Height - this.popup.HeightRequest) / 2)
            );
        }

        /// <summary>
        /// Shows the pop-up in the given x and y positions with the given width and height.
        /// </summary>
        /// <param name="popupView">The pop-up view.</param>
        /// <param name="xConstraint">X constraint.</param>
        /// <param name="yConstraint">Y constraint.</param>
        /// <param name="widthConstraint">Optional width constraint.</param>
        /// <param name="heightConstraint">Optional height constraint.</param>
        public void ShowPopup(View popupView, Constraint xConstraint, Constraint yConstraint, Constraint widthConstraint = null, Constraint heightConstraint = null)
        {
            Dismiss();
            this.popup = popupView;

           this.popUpContentLayout.Children.Add(this.popup, xConstraint, yConstraint, widthConstraint, heightConstraint);

            this.popUpContentLayout.ForceLayout();
        }


        /// <summary>
        /// Shows the pop-up relative to another view.
        /// </summary>
        /// <param name="popupView">The pop-up view.</param>
        /// <param name="relativeView">The relative view for displaying the pop-up.</param>
        /// <param name="location">The location.</param>
        /// <param name="paddingX">The padding x.</param>
        /// <param name="paddingY">The padding y.</param>
        public void ShowPopup(View popupView, View relativeView, PopupLocation location, float paddingX = 0, float paddingY = 0)
        {
            Dismiss();
            this.popup = popupView;

            Constraint constraintX = null, constraintY = null;

            switch (location)
            {
                case PopupLocation.Bottom:
                    constraintX = Constraint.RelativeToParent(parent => relativeView.X + (relativeView.Width - this.popup.WidthRequest) / 2);
                    constraintY = Constraint.RelativeToParent(parent => parent.Y + relativeView.Y + relativeView.Height + popup.HeightRequest + paddingY);
                    break;
                case PopupLocation.Top:
                    constraintX = Constraint.RelativeToParent(parent => relativeView.X + (relativeView.Width - this.popup.WidthRequest) / 2);
                    constraintY = Constraint.RelativeToParent(parent => parent.Y + relativeView.Y - paddingY);
                    break;
                case PopupLocation.Left:
                    constraintX = Constraint.RelativeToView(relativeView, (parent, view) => ((view.X + view.Height / 2) - parent.X) + this.popup.HeightRequest / 2);
                    constraintY = Constraint.RelativeToView(relativeView, (parent, view) => parent.Y + view.Y + view.Width + paddingY);
                    break;
                case PopupLocation.Right:
                    constraintX = Constraint.RelativeToView(relativeView, (parent, view) => ((view.X + view.Height / 2) - parent.X) + this.popup.HeightRequest / 2);
                    constraintY = Constraint.RelativeToView(relativeView, (parent, view) => parent.Y + view.Y - this.popup.WidthRequest - paddingY);
                    break;
            }

            this.ShowPopup(popupView, constraintX, constraintY);
        }

        /// <summary>
        /// Dismisses the pop-up.
        /// </summary>
        public void Dismiss()
        {
            if (this.popup != null)
            {
                this.popUpContentLayout.Children.Remove(this.popup);
                this.popup = null;
            }

            this.popUpContentLayout.InputTransparent = false;

            if (this.content != null)
            {
                this.content.InputTransparent = false;
            }
        }
    }
}
